/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _DAQ_STRUCT_H_
#define _DAQ_STRUCT_H_

#include <vector>
#include <string>
#include "DAQ_enum.h"
#include "Data_struct.h"

/**
 *\file DAQ_struct.h
*
 *\date  Created on: 10-mar-2009
 *\author  Author: Claudio Salvadori
 *\brief General structures used by DAQ*/
 
/**
*\struct SMData
* \brief Structure of metadata elements
*/
struct SMData{
	std::string 	md_Name;	/**< Element name*/
	unsigned long 	md_Pos;		/**< Position of the element in the packet (in bytes)*/
	unsigned long 	md_Size;	/**< Size of the element (in bytes)*/
	var_type	md_Type;	/**< Type of the element*/
	endianess_type	md_End;		/**< Endianess*/
	
};

/**
*\struct Srs_232
* \brief Structure of RS-232 Preset
*/
struct Srs_232{
	unsigned int	rs_Baud;	/**< Baud rate*/
	unsigned int	rs_DBits;	/**< Number of data bits*/
	std::string	rs_Parity; 	/**< Parity bit*/
	float		rs_StopB;	/**< Number of stop bits*/
};

/**
*\struct SFile
* \brief Structure of File
*/
struct SFile{
	unsigned long	f_Header;	/**< Header length (in bytes)*/
	unsigned long	f_Entry_S;	/**< Number of entry (start byte, 0-based)*/
	unsigned int	f_Entry_L; 	/**< Number of entry (length in bytes)*/
	unsigned long	f_Trailer;	/**< Trailer start (0-based)*/
};
/*******************************************************************************/
/*   ADD HERE A STRUCT TO DESCRIBE THE ADDED DEVICE PRESET, IF IT'S NEEDED     */
/*******************************************************************************/

/**
*\struct SHandshaking
* \brief Structure of handshaking packets
*/
struct SHandshaking{
	unsigned char*		h_Pkt;		/**< Packet to send*/
	unsigned int		h_Size_P;	/**< Packet to send size*/
	unsigned char*		h_Ack;		/**< Ack value*/
	unsigned int		h_Size_A;	/**< Ack size*/
	unsigned char*		h_Nack; 	/**< Nack value*/
	unsigned int		h_Size_N;	/**< Nack size*/
};

/**
*\struct SOption
* \brief Structure of steam option: syncro character
*/
struct SOption{
	decoder_type		o_Dec;		/**< Decoder type*/
	unsigned char		o_Start;	/**< Packet start character*/
	unsigned char		o_End;		/**< Packet end character*/
	unsigned char		o_Esc; 		/**< Escape character*/
};

/**
*\struct SDef
* \brief Structure for stream definition
*/
struct SDef{
	stream_type						s_Type;		/**< Stream type*/
	std::string						s_Name;		/**< Stream name*/
	unsigned int						s_Chan; 	/**< Logical channel number (from stream_info attribute in xml)*/
	unsigned int						s_Sock_Port;	/**< Socket port number (from stream_info attribute in xml)*/
	unsigned long						s_Size;		/**< Packet total size*/
	struct Srs_232*						s_Preset;	/**< RS-232 presets (baud rate, number of data bit, ...) \see Srs_232*/
	struct SFile*						s_File;		/**< File structure presets \see SFile*/
	/**********************************************************************************************************************************************/
	/*                        ADD HERE AN INSTANCE OF A NEW STRUCT TO DESCRIBE THE NEW STREAM, IF IT'S NECESSARY                                  */
	/**********************************************************************************************************************************************/

	handshake_style						s_Style;	/**< Handshake style \see handshake_style*/
	std::vector<struct SHandshaking>*			s_Handshk;	/**< Handshaking packets to start communication \see SHandshaking*/
	struct SOption*						s_Option;	/**< Character of syncro and of escape \see SOption*/
};

/**
*\struct SMessage
* \brief Structure of message
*/
struct SMessage{
	unsigned long	me_Size;	/**< Size of the message*/
	unsigned char*	me_Data; 	/**< Pointer to the message*/
};

#endif
	
