/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

 /**
 *\file CTable.cpp
 *\brief CTable class method development.
 */

#include "CTable.h"
#include <myUtils.h>
#include <iostream>
#include <iomanip>

IRenderer::Factory registerFactoryT("table", CTable::createObj);

class TableException : public std::exception {
public:
        TableException(const char *msg) throw() : errMessage(msg) { } ;
        ~TableException(void) throw() { } ;
        const char *what(void) const throw() { return errMessage.c_str(); };
private:
        std::string errMessage;
       	TableException(void) throw();
};

IRenderer* CTable::createObj(const void* node){
	return new CTable((const DOMNode*)node);
}

CTable::CTable(const DOMNode* node){
	parser_ = new CXMLParser;
	tablePreset(node);

	file_.open(name_.c_str());
}

CTable::~CTable(){
	if(file_) file_.close();
		
	if(tnode_) delete tnode_;
	if(parser_) delete parser_;	
		
}

void CTable::tablePreset(const DOMNode* node){
	char ret[256];

	if(!parser_->FindNodeRootat((DOMNode*)node, (char*)"table_preset", tnode_)){
		throw TableException("table_preset node isn't found");
	}else{
		if (!parser_->GetAttribute(tnode_, (char*)"table_name", (char*)ret)){
			throw TableException("table_name attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Table name returned: %s\n", ret);
			#endif
			name_ = ret;		/* Table name*/
			IRenderer::checkName(name_);
		}
				
	}	
}

void CTable::render(void* data, unsigned long size){
	
	switch(vType_){
		case T_INT8:
		file_ << *((char*)data) << std::endl;
		break;
		
		case T_UINT8: 
		file_ << *((unsigned char*)data) << std::endl;
		break;
			
		case T_INT16:
		file_ << *((short*)data) << std::endl;
		break;
			
		case T_UINT16:
		file_ << *((unsigned short*)data) << std::endl;
		break;
			
		case T_INT32:
		file_ << *((int*)data) << std::endl;
		break;
		
		case T_UINT32:;
		file_ << *((unsigned int*)data) << std::endl;
		break;
			
		case T_INT64:
		file_ << *((long*)data) << std::endl;
		break;
			
		case T_UINT64:
		file_ << *((unsigned long*)data) << std::endl;
		break;
		
		case T_FLOAT32:
		file_ << *((float*)data) << std::endl;
		break;
			
		case T_FLOAT64:
		file_ << *((double*)data) << std::endl;
		break;
		
		default:
			break;
	}
}


